/*
** titles.c -- Title making routines
**
** Title making routines are available, but in order to keep the size
** of the movie viewer to a minimum, the title routines are all within
** precompiler conditionals called TITLE.
** To compile in the title making routines, #define TITLE.
**
** Michael J. Gourlay
**
** NO WARRANTEES, EXPRESS OR IMPLIED.
*/

#include <fcntl.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "showtime.h"
#include "video.h"
#include "pgmcload.h"
#include "file.h"

/* ====================================================================== */

#ifdef TITLE

/*
** TITLE : title generation routines.
**
** These routines are used to generate titles for movies,
** and save the generated titles to files for use in movies.
**
** In order to keep the movie viewer compact and quick,
** the TITLE code is compilation conditional.
** The intention is to have this source code generate two programs,
** one which is small and faster (the movie viewer)
** and one which is functionally a superset of the viewer (the title generator)
*/

/*
** initialize_font : register fonts
** font_dir is the font directory and filename specifier
** font_script is the font script output filename
** fs_fp is the font script output file pointer
*/
int
initialize_font(char *font_dir, char *font_script, FILE **fs_fp)
{

#if defined(MSC)
  /* If font_dir is not set, look in the environment for a value */
  if(font_dir==NULL) font_dir=getenv("FONTDIR");
  /* If font_dir is not set, set a default value */
  if(font_dir==NULL) font_dir="*.FON";

  /* register fonts */
  if(_registerfonts(font_dir) < 0) {
    logmesg("\ninit_font: could not register fonts\n");
    reset_display();
    bye_bye(1);
  }

  /* Open the script output file */
  /* (Used to help the user generate a script from the interactive commands) */
  if(font_script!=NULL) {
    if((*fs_fp=fopen(font_script, "ac"))==NULL) {
      logmesg("\ninit_font: could not open script file %s for writing\n",
              font_script);
      reset_display();
      bye_bye(1);
    }
  }
#endif /* MSC */

  return(0);
}

/* ---------------------------------------------------------------------- */

/*
** write_image_header : write the PGM header for an image
** x1, y1 are the upper left corner
** x2, y2 are the lower right corner
** fiod is the output file descriptor
*/
int
write_image_header(UWORD x1, UWORD y1, UWORD x2, UWORD y2, FIO_T fiod)
{
  long num_rows, num_cols;
  char header[40];

  num_rows = (y2-y1+1);
  num_cols = (x2-x1+1);

  sprintf(header, "P5\n%lu %lu\n%i\n", num_cols, num_rows, 255);
#ifdef FAST_IO
  write(fiod, header, strlen(header));
#else
  fwrite(header, 1, strlen(header), fiod);
#endif /* FAST_IO */

#ifdef FAST_IO
  write(fiod, &num_rows, 4); /* num_rows */
  write(fiod, &num_cols, 4); /* num_cols */
#else
  fwrite(&num_rows, 4, 1, fiod); /* num_rows */
  fwrite(&num_cols, 4, 1, fiod); /* num_cols */
#endif /* FAST_IO */

  return(0);
}

/* ---------------------------------------------------------------------- */

/*
** write_image : capture a region and write a quasi-PGM image to a file
** x1, y1 are the upper left corner
** x2, y2 are the lower right corner
** ofn is the output filename
*/
int
write_image(UWORD x1, UWORD y1, UWORD x2, UWORD y2, char *ofn)
{
  long          image_size;
  unsigned char *image;
  long          header_size;
  long          image_bytes;
  FIO_T         fiod;

#if defined(MSC)
  image_size  = _imagesize(x1, y1, x2, y2);
  image_bytes = (x2-x1+1)*(y2-y1+1);
  header_size = image_size - image_bytes;

  if((image=(unsigned char *) malloc(image_size))==NULL) {
    logmesg("write_image: could not allocate %li bytes\n", image_size);
    return(1);
  }

  _getimage(x1, y1, x2, y2, image);
#endif /* MSC */

  /* Remove old file if it exists */
  unlink(ofn);

#ifdef FAST_IO
  if((fiod=open(ofn, O_CREAT | O_WRONLY | O_BINARY))== -1)
#else
  if((fiod=fopen(ofn, "wb"))==NULL)
#endif /* FAST_IO */
  {
    logmesg("write_image: couldn't open %s for write\n", ofn);
    return(1);
  }

  write_image_header(x1, y1, x2, y2, fiod);
#ifdef FAST_IO
  write(fiod, (void *)(image + header_size), image_bytes);
#else
  fwrite((void *)(image + header_size), 1, image_bytes, fiod);
#endif /* FAST_IO */

  FIO_CLOSE(fiod);

  return(0);
}
#endif /* TITLE */

